/****************************************************************************
 Module
   IO_Service.c

 Revision
   1.0.1

 Description
   IO Service for handling PWM for thrust fans, lift fan, and servo motor. Implements
	 a PD control loop to match the user's desired rotational velocity.
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include <stdint.h>
#include <stdbool.h>
#include "bitdefs.h"

#include "inc/hw_gpio.h"
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_pwm.h"

#include "driverlib/sysctl.h"
#include "driverlib/pin_map.h"
#include "driverlib/gpio.h"
#include "driverlib/pwm.h"

#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_Port.h"

#include "IO_Service.h"
#include "SPI.h"
#include "Tiva_XBee_Comm.h"

/*----------------------------- Module Defines ----------------------------*/
#define IO_SERVICE_DEBUG_FLAG 0
#define IO_SERVICE_DEBUG if(IO_SERVICE_DEBUG_FLAG) printf

//drive motor defines
#define PWM_TICKS_PER_S 40000000/32
#define LIFT_FREQUENCY 1000
#define MOTOR_FREQUENCY 10000
#define SERVO_FREQUENCY 50
#define BITS_PER_NIBBLE 4

#define MAX_POWER 45
#define MIN_POWER 10
#define MAX_REVERSE_POWER -45
#define MAX_DIRECTION 128

#define BRAKE 0
#define DRAG 1
#define DRIVE 2
#define DRAG_DUTY 80

#define UP 0
#define DOWN 1
#define UP_DUTY 7
#define DOWN_DUTY 3

#define ALL_BITS (0xff<<2)

#define NUM_BALLOON_SAMPLES 10

#define CONTROL_TIMEOUT 50
#define BALLOON_TIMEOUT 50

//PID CONTROL
#define PGAIN 80/128
#define DGAIN 4
#define THRUST_SCALAR 50/128

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this service.They should be functions
   relevant to the behavior of this service
*/

void InitDrivePWM(void);
void InitLiftPWM(void);
void InitMindControlIndicatorPWM(void);
void InitLEDs(void);
void InitBalloonMonitorInput(void);

void SetDrivePWMDuty(int leftDuty, int rightDuty);
void SetLights(uint8_t frontrear, uint8_t onoff);

void RunControlLaw(void);
void SetServoPosition(uint8_t Position);
int16_t Cap(int16_t Duty);


/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;

static int16_t RawThrust = 0;
static int16_t TargetRotation = 0;

static int16_t CurrentRotation = 0;

static uint8_t BalloonStatusArray[NUM_BALLOON_SAMPLES] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

static bool isBraked = true;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitIO_Service

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any
     other required initialization for this service
 Notes

 Author
     B. Allen Freund, 05/15/15, 15:15
****************************************************************************/
bool InitIO_Service ( uint8_t Priority )
{
	MyPriority = Priority;

	InitDrivePWM();
	InitLiftPWM();
	InitBalloonMonitorInput();
	InitMindControlIndicatorPWM();
	//InitLEDs();
	SetLights(FRONT, ON);
	SetLights(REAR, ON);
	
	SetDrivePWMDuty(0,0);

	ES_Timer_InitTimer(BALLOON_TIMER, BALLOON_TIMEOUT);
	ES_Timer_InitTimer(PID_TIMER, CONTROL_TIMEOUT);

	return true;
}

/****************************************************************************
 Function
     PostIO_Service

 Parameters
     EF_Event ThisEvent ,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
		 B. Allen Freund, 05/15/15, 15:15
****************************************************************************/
bool PostIO_Service( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunIO_Service

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes

 Author
   B. Allen Freund, 05/15/15, 15:15
****************************************************************************/
ES_Event RunIO_Service( ES_Event ThisEvent )
{
	ES_Event ReturnEvent;
	ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
	if (ThisEvent.EventType == ES_TIMEOUT) {
		if (ThisEvent.EventParam == PID_TIMER) {
			RunControlLaw();
			ES_Timer_InitTimer(PID_TIMER, CONTROL_TIMEOUT);
		} else if (ThisEvent.EventParam == BALLOON_TIMER) {
			for (int i = 1; i < NUM_BALLOON_SAMPLES; i++) {
				BalloonStatusArray[i] = BalloonStatusArray[i - 1];
			}
			BalloonStatusArray[0] = ((HWREG(GPIO_PORTD_BASE + GPIO_O_DATA + ALL_BITS) & GPIO_PIN_0) == GPIO_PIN_0);
			ES_Timer_InitTimer(BALLOON_TIMER, BALLOON_TIMEOUT);
		}
	}

	if (ThisEvent.EventType == EV_NEW_GYRO) {
		CurrentRotation = QueryGyro();
	}

	return ReturnEvent;
}

/****************************************************************************
 Function
    SetLiftFan

 Parameters
   uint8_t mode : a constant indicating whether to drive the lift fan
   full strength, half strength, or not at all

 Returns
   None

 Description
   Allows for different lift modes to be set

 Author
   C. Paul Bennett, 05/17/15, 13:24
****************************************************************************/
void SetLiftFan(uint8_t mode) {
	if (mode == BRAKE) { // set duty to 0
		IO_SERVICE_DEBUG("Brake On\r\n");
		isBraked = true;
		HWREG(GPIO_PORTB_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT2HI;
	} else if (mode == DRAG) { //  set duty to 50
		IO_SERVICE_DEBUG("Brake Off\r\n");
		isBraked = false;
		HWREG(GPIO_PORTB_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT2HI;
	} else if (mode == DRIVE) { // set duty to 100
		IO_SERVICE_DEBUG("Brake Off\r\n");
		isBraked = false;
		HWREG(GPIO_PORTB_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT2HI;
	}
}

/****************************************************************************
 Function
    UpdateOrientation

 Parameters
   int8_t NewThrust:		A number between -127 and 128 indicating how much to drive
   								the hovercraft forward/backward

   int8_t NewRotation:	A number between -127 and 128 indicating how much to drive
   								the hovercraft left/right

 Returns
   None

 Description
   Updates the values that the control loop works on

 Author
   C. Paul Bennett, 05/17/15, 13:24
****************************************************************************/
void UpdatePIDData(int8_t NewThrust, int8_t NewRotation)
{
	RawThrust = (int16_t)NewThrust;
	TargetRotation = (int16_t)NewRotation;
	//IO_SERVICE_DEBUG("NewThrust: %8i, NewRotation: %8i \r\n", NewThrust, NewRotation);
}

/****************************************************************************
 Function
    SetServoPosition

 Parameters
   None

 Returns
   The status of the balloon monitor - 1 if popped, 0 if unpopped

 Author
   C. Paul Bennett, 05/17/15, 13:24
****************************************************************************/
void SetServoPosition(uint8_t Position)
{
	if (Position == UP) {
		//IO_SERVICE_DEBUG("UP\r\n");
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM7EN;
		HWREG( PWM0_BASE + PWM_O_3_CMPB) = ((((PWM_TICKS_PER_S / SERVO_FREQUENCY) - 1) >> 1) * UP_DUTY) / 100;
	} else if (Position == DOWN) {
		//IO_SERVICE_DEBUG("DOWN\r\n");
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM7EN;
		HWREG( PWM0_BASE + PWM_O_3_CMPB) = ((((PWM_TICKS_PER_S / SERVO_FREQUENCY) - 1) >> 1) * DOWN_DUTY) / 100;
	}
}

/****************************************************************************
 Function
    QueryBalloonMonitor

 Parameters
   None

 Returns
   The status of the balloon monitor - 1 if popped, 0 if unpopped

 Author
   C. Paul Bennett, 05/17/15, 13:24
****************************************************************************/
uint8_t QueryBalloonMonitor(void)
{
	uint8_t Sum = 0;
	for (int i = 0; i < NUM_BALLOON_SAMPLES; i++) {
		Sum += BalloonStatusArray[i];
	}
	return Sum / NUM_BALLOON_SAMPLES;
}

/***************************************************************************
 private functions
 ***************************************************************************/
// Uses PD0
void InitBalloonMonitorInput(void) {
	// Enable the clock to Port D
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R3;

	// make sure that the Port D module clock has gotten going
	while ((HWREG(SYSCTL_PRGPIO) & SYSCTL_PRGPIO_R3) != SYSCTL_PRGPIO_R3)
		;

	// Set pin 0 on Port D as digital I/O
	HWREG(GPIO_PORTD_BASE + GPIO_O_DEN) |= GPIO_PIN_0;

	// make pin 0 on Port D into an input
	HWREG(GPIO_PORTD_BASE + GPIO_O_DIR) &= ~GPIO_PIN_0;
}

/****************************************************************************
 Function
    RunControlLaw

 Parameters
   None

 Returns
   Computes the error in rotational velocities from the gyroscope data and user input
	 and determines the appropriate duty cycles of the thrusters. PGAIN and DGAIN can 
	 be modified to adjust the response time.

 Author
		 B. Allen Freund, 05/15/15, 15:15
****************************************************************************/
void RunControlLaw(void) {
	if (QueryXBeeState() == Unpaired || isBraked) {
		SetDrivePWMDuty(0, 0);
	} else {
		static int counter = 0;
		if(RawThrust >=-10) {
			static int last_error;
			int error = TargetRotation - CurrentRotation;
			int derivative = (error - last_error);
			last_error = error;
			int leftDuty = RawThrust * THRUST_SCALAR - error * PGAIN;
			int rightDuty = RawThrust * THRUST_SCALAR + error * PGAIN;
			leftDuty = Cap(leftDuty);
			rightDuty = Cap(rightDuty);
			if(++counter%10 == 1) {
				IO_SERVICE_DEBUG("Target: %8i, Current: %8i, Error: %8i, Left Duty: %i, Right Duty: %i\r\n", TargetRotation, CurrentRotation, error, leftDuty, rightDuty);
			}
			SetDrivePWMDuty(leftDuty, rightDuty);
		} else {
			SetDrivePWMDuty(MAX_REVERSE_POWER, MAX_REVERSE_POWER);
		}
	}
}

int16_t Cap(int16_t Duty) {
	if (Duty > MAX_POWER) return MAX_POWER;
	if (Duty < MAX_REVERSE_POWER) return MAX_REVERSE_POWER;
	return Duty;
}


void SetDrivePWMDuty(int leftDuty, int rightDuty) {
	// Direction is 1 if duty is greater than 0, 0 otherwise
	uint8_t leftDir = (leftDuty >= 0);
	uint8_t rightDir = (rightDuty >= 0);

	// Take the absolute value of the duty cycle
	rightDuty = ((rightDuty >= 0) ? rightDuty : -rightDuty);
	leftDuty = ((leftDuty >= 0) ? leftDuty : -leftDuty);

	if (leftDir) leftDuty = 100 - leftDuty;
	if (rightDir) rightDuty = 100 - rightDuty;

	//IO_SERVICE_DEBUG("Left Duty: %i, Right Duty: %i, LeftDir: %i, RightDir: %i\r\n", leftDuty, rightDuty, leftDir, rightDir);


	if (leftDuty == 0) {
		// Set the Duty cycle on A to 0% by programming the compare value
		// to 0. However, since the CmpADn action (set to one) wins, we also
		// need to disable the output
		HWREG( PWM0_BASE + PWM_O_0_CMPA) = 0;
		HWREG( PWM0_BASE + PWM_O_ENABLE) &= ~PWM_ENABLE_PWM0EN;
	} else if (leftDuty > 0 && leftDuty < 100) {
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM0EN;
		HWREG( PWM0_BASE + PWM_O_0_CMPA) = ((((PWM_TICKS_PER_S / MOTOR_FREQUENCY) - 1) >> 1) * leftDuty) / 100;
	} else if (leftDuty == 100) {
		// Set the Duty cycle on A to 100% by programming the compare value
		// to the load value. Since the CmpADn action (set to one) wins, we get 100%
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM0EN;
		HWREG( PWM0_BASE + PWM_O_0_CMPA) = HWREG( PWM0_BASE + PWM_O_0_LOAD);
	}

	//set the left direction bit to high if moving forward, low if moving backward
	if (!leftDir) {
		HWREG(GPIO_PORTC_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT7HI;
	} else {
		HWREG(GPIO_PORTC_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT7HI;
	}

	if (rightDuty == 0) {
		// Set the Duty cycle on B  to 0% by programming the compare value
		// to 0. However, since the CmpBDn action (set to one) wins, we also
		// need to disable the output
		HWREG( PWM0_BASE + PWM_O_0_CMPB ) = 0;
		HWREG( PWM0_BASE + PWM_O_ENABLE) &= ~PWM_ENABLE_PWM1EN;
	} else if (rightDuty > 0 && rightDuty < 100) {
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM1EN;
		HWREG( PWM0_BASE + PWM_O_0_CMPB) = ((((PWM_TICKS_PER_S / MOTOR_FREQUENCY) - 1) >> 1) * rightDuty) / 100;
	} else if (rightDuty == 100) {
		// Set the Duty cycle on B to 100% by programming the compare value
		// to the load value. Since the CmpBDn action (set to one) wins, we get 100%
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM1EN;
		HWREG( PWM0_BASE + PWM_O_0_CMPB) = HWREG( PWM0_BASE + PWM_O_0_LOAD);
	}

	//set the right direction bit to high if moving forward, low if moving backward
	if (!rightDir) {
		HWREG(GPIO_PORTC_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT6HI;
	} else {
		HWREG(GPIO_PORTC_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT6HI;
	}
}

void SetLights(uint8_t frontrear, uint8_t onoff) {
	/*
	switch (frontrear) {
		case FRONT:
			if (onoff == OFF) {
				HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~(BIT1HI | BIT2HI);
			} else {
				printf("front lights on\r\n");
				HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT1HI | BIT2HI;
			}
		break;
		case REAR:
			if (onoff == OFF) {
				HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT3HI;
				HWREG(GPIO_PORTE_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT1HI;
			} else {
				printf("rear lights on\r\n");
				HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT3HI;
				HWREG(GPIO_PORTE_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT1HI;
			}
		break;
	}*/
}



void InitDrivePWM(void) {
	// start by enabling the clock to the PWM Module (PWM0)
	HWREG(SYSCTL_RCGCPWM) |= SYSCTL_RCGCPWM_R0;

	// enable the clock to Port B and C
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R1 | SYSCTL_RCGCGPIO_R2;

	// Select the PWM clock as System Clock/32
	HWREG(SYSCTL_RCC) = (HWREG(SYSCTL_RCC) & ~SYSCTL_RCC_PWMDIV_M) |
	                    (SYSCTL_RCC_USEPWMDIV | SYSCTL_RCC_PWMDIV_32);

	// make sure that the PWM module clock has gotten going
	while ((HWREG(SYSCTL_PRPWM) & SYSCTL_PRPWM_R0) != SYSCTL_PRPWM_R0)
		;

	// disable the PWM while initializing
	HWREG( PWM0_BASE + PWM_O_0_CTL ) = 0;

	// program generator A to go to 0 at rising comare A, 1 on falling compare A
	HWREG( PWM0_BASE + PWM_O_0_GENA) =
	    (PWM_0_GENA_ACTCMPAU_ZERO | PWM_0_GENA_ACTCMPAD_ONE );

	// program generator B to go to 0 at rising comare B, 1 on falling compare B
	HWREG( PWM0_BASE + PWM_O_0_GENB) =
	    (PWM_0_GENB_ACTCMPBU_ZERO | PWM_0_GENB_ACTCMPBD_ONE );

	// Set the PWM period. Since we are counting both up & down, we initialize
	// the load register to 1/2 the desired total period
	HWREG( PWM0_BASE + PWM_O_0_LOAD) = ((PWM_TICKS_PER_S / MOTOR_FREQUENCY) - 1) >> 1;

	// Set the initial Duty cycle on A to 0
	HWREG( PWM0_BASE + PWM_O_0_CMPA) = 0;

	// Set the initial Duty cycle on B to 0
	HWREG( PWM0_BASE + PWM_O_0_CMPB) = 0;

	// set changes to the PWM output Enables to be locally syncronized to a
	// zero count
	HWREG(PWM0_BASE + PWM_O_ENUPD) =  (HWREG(PWM0_BASE + PWM_O_ENUPD) &
	                                   ~(PWM_ENUPD_ENUPD0_M | PWM_ENUPD_ENUPD1_M)) |
	                                  (PWM_ENUPD_ENUPD0_LSYNC | PWM_ENUPD_ENUPD1_LSYNC);

	// now configure the Port B pins to be PWM outputs
	// start by selecting the alternate function for PB6 & 7
	HWREG(GPIO_PORTB_BASE + GPIO_O_AFSEL) |= (BIT7HI | BIT6HI);

	// now choose to map PWM to those pins, this is a mux value of 4 that we
	// want to use for specifying the function on bits 6 & 7
	HWREG(GPIO_PORTB_BASE + GPIO_O_PCTL) =
	    (HWREG(GPIO_PORTB_BASE + GPIO_O_PCTL) & 0x00ffffff) + (4 << (7 * BITS_PER_NIBBLE)) + (4 << (6 * BITS_PER_NIBBLE));

	// Enable pins C6 & 7 (Direction) and 6 & 7 (PWM) on Port B for digital I/O
	HWREG(GPIO_PORTB_BASE + GPIO_O_DEN) |= (BIT6HI | BIT7HI);
	HWREG(GPIO_PORTC_BASE + GPIO_O_DEN) |= (BIT6HI | BIT7HI);

	// make pins C6 & 7 (Direction) and 6 & 7 (PWM) on Port B into outputs
	HWREG(GPIO_PORTB_BASE + GPIO_O_DIR) |= (BIT6HI | BIT7HI);
	HWREG(GPIO_PORTC_BASE + GPIO_O_DIR) |= (BIT6HI | BIT7HI);

	// set the up/down count mode and enable the PWM generator
	HWREG(PWM0_BASE + PWM_O_0_CTL) |= (PWM_0_CTL_MODE | PWM_0_CTL_ENABLE);
}



// Pin B5, Module 0, Generator 1 (B), PWM 3
void InitLiftPWM(void) {
	// Enable pin 5 on Port B for digital I/O
	HWREG(GPIO_PORTB_BASE + GPIO_O_DEN) |= BIT2HI;

	// make pins 5 on Port B into outputs
	HWREG(GPIO_PORTB_BASE + GPIO_O_DIR) |= BIT2HI;
}

//Init pins D<1:2> as head lights and D3, E1 as brake lights
void InitLEDs(void) {
	// enable the clock to Ports E and F
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R3 | SYSCTL_RCGCGPIO_R4;
	
	// enable pins 1-3 on Port E, and pin 1 on Port F for digital I/O
	HWREG(GPIO_PORTD_BASE + GPIO_O_DEN) |= GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3;
	HWREG(GPIO_PORTE_BASE + GPIO_O_DEN) |= GPIO_PIN_1;

	// make pins 1-3 on Port E, and pin 1 on Port F into outputs
	HWREG(GPIO_PORTD_BASE + GPIO_O_DIR) |= GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3;
	HWREG(GPIO_PORTE_BASE + GPIO_O_DIR) |= GPIO_PIN_1;
}

// Pin C5, Module 0, Generator 3 (A), PWM 6
void InitMindControlIndicatorPWM(void) {
	// start by enabling the clock to the PWM Module (PWM0)
	HWREG(SYSCTL_RCGCPWM) |= SYSCTL_RCGCPWM_R0;

	// enable the clock to Port C
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R2;

	// Select the PWM clock as System Clock/32
	HWREG(SYSCTL_RCC) = (HWREG(SYSCTL_RCC) & ~SYSCTL_RCC_PWMDIV_M) |
	                    (SYSCTL_RCC_USEPWMDIV | SYSCTL_RCC_PWMDIV_32);

	// make sure that the PWM module clock has gotten going
	while ((HWREG(SYSCTL_PRPWM) & SYSCTL_PRPWM_R0) != SYSCTL_PRPWM_R0)
		;

	// disable the PWM while initializing
	HWREG( PWM0_BASE + PWM_O_3_CTL ) = 0;

	// program generator A to go to 0 at rising comare A, 1 on falling compare A
	HWREG( PWM0_BASE + PWM_O_3_GENB) = (PWM_3_GENB_ACTCMPBU_ZERO | PWM_3_GENB_ACTCMPBD_ONE );

	// Set the PWM period. Since we are counting both up & down, we initialize
	// the load register to 1/2 the desired total period
	HWREG( PWM0_BASE + PWM_O_3_LOAD) = ((PWM_TICKS_PER_S / SERVO_FREQUENCY) - 1) >> 1;

	// Set the initial Duty cycle on B to 0
	HWREG( PWM0_BASE + PWM_O_3_CMPB) = 0;

	// Set the initial Duty cycle on A to 0
	HWREG( PWM0_BASE + PWM_O_3_CMPA) = 0;

	// set changes to the PWM output Enables to be locally syncronized to a
	// zero count
	HWREG(PWM0_BASE + PWM_O_ENUPD) =  (HWREG(PWM0_BASE + PWM_O_ENUPD) & ~PWM_ENUPD_ENUPD7_M) |
	                                  (PWM_ENUPD_ENUPD7_LSYNC);

	// enable the PWM output
	HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM7EN;

	// now configure the Port C pins to be PWM outputs
	// start by selecting the alternate function for PC5
	HWREG(GPIO_PORTC_BASE + GPIO_O_AFSEL) |= GPIO_PIN_5;

	// now choose to map PWM to those pins, this is a mux value of 4 that we
	// want to use for specifying the function on bits 5
	HWREG(GPIO_PORTC_BASE + GPIO_O_PCTL) =
	    (HWREG(GPIO_PORTC_BASE + GPIO_O_PCTL) & 0xff0fffff)
	    + (4 << (5 * BITS_PER_NIBBLE));

	// Enable pin 4 on Port C for digital I/O
	HWREG(GPIO_PORTC_BASE + GPIO_O_DEN) |= GPIO_PIN_5;

	// make pin 4 on Port C into an output
	HWREG(GPIO_PORTC_BASE + GPIO_O_DIR) |= GPIO_PIN_5;

	// set the up/down count mode and enable the PWM generator
	HWREG(PWM0_BASE + PWM_O_3_CTL) |= (PWM_3_CTL_MODE | PWM_3_CTL_ENABLE);

	IO_SERVICE_DEBUG("PWM Init Done \r\n");
}


/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/
