/****************************************************************************
 Module
   Popping_SM.c

 Revision
   1.0.1

 Description

****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/

#include "PopperSM.h"

#include "ES_Framework.h"
#include "ES_Timers.h"
#include "ES_Queue.h"
#include "ES_Configure.h"
#include "ES_Port.h"

#include <stdio.h>
#include <stdint.h>

#include "inc/hw_memmap.h"
#include "inc/hw_timer.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "inc/hw_uart.h"
#include "inc/hw_gpio.h"
#include "inc/hw_pwm.h"
#include "bitdefs.h"

#include "driverlib/gpio.h"
#include "driverlib/pwm.h"
#include "driverlib/sysctl.h"
#include "driverlib/pin_map.h"

#include "IO_Service.h"
#include "SPI.h"
#include "Tiva_XBee_Comm.h"

/*----------------------------- Module Defines ----------------------------*/
#define ALL_BITS (0xff<<2)

// 40,000 ticks per mS assumes a 40Mhz clock, we will use SysClk/32 for PWM
#define PWMTicksPerMS 40000/32

// set 200 Hz frequency so 5mS period
#define PeriodInMS 4
#define PWM_TICKS_PER_S 40000000/32
#define POPPER_FREQUENCY 10000
#define BITS_PER_NIBBLE 4

// Timer timeouts
#define RAMP_TIME 500
#define FORWARD_TIME 600
#define BACKWARD_TIME 400
#define	POPPER_TIME 50
#define	ACTIVE_POPPING 1500
#define	POP_LIMIT	2000

// PWM Duty Cycles
#define POP_SPEED 99
#define RACK_SPEED 50

// PWM Directions
#define FORWARD 0
#define BACKWARD 1

#define POPPER_ON 1
#define POPPER_OFF 0
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
void SetPinionDuty(uint8_t duty, uint8_t direction);
void SetPopperMode(uint8_t mode);
void InitPopperPWM(void);
/*---------------------------- Module Variables ---------------------------*/

// with the introduction of Gen2, we need a module level Priority var as well
static PoppingState_t CurrentState;

static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitPopping_SM

 Parameters

 Returns

 Description

 Notes

 Author
     Team 13, 10/23/11, 18:55
****************************************************************************/
bool InitPopping_SM (uint8_t Priority)
{
	MyPriority = Priority;
	InitPopperPWM();
	CurrentState = Idle;

	return true;
}

/****************************************************************************
 Function
     PostPopping_SM

 Parameters
     EF_Event ThisEvent ,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostPopping_SM( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunPopping_SM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   Team 13, 01/15/12, 15:23
****************************************************************************/
ES_Event RunPopping_SM( ES_Event CurrentEvent )
{
	ES_Event ReturnEvent;
	ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

	switch ( CurrentState ) {

		
	case Idle:
		if (CurrentEvent.EventType == EV_POP) {
			printf("ramping popper\r\n");
			CurrentState = RampUp;
			SetPopperMode(POPPER_ON);
			ES_Timer_InitTimer(POPPER_TIMER, RAMP_TIME);
		}
		break;
		
	case RampUp:
		if (CurrentEvent.EventType == ES_TIMEOUT && CurrentEvent.EventParam == POPPER_TIMER) {
			printf("Send the rack forward\r\n");
			CurrentState = RackForward;
			SetPinionDuty(RACK_SPEED, FORWARD);
			ES_Timer_InitTimer(POPPER_TIMER, FORWARD_TIME);
		}
		break;

	case RackForward:
		if (CurrentEvent.EventType == ES_TIMEOUT && CurrentEvent.EventParam == POPPER_TIMER) {
			printf("Send the rack backward!\r\n");
			CurrentState = RackBackward;
			SetPinionDuty(RACK_SPEED, BACKWARD);
			ES_Timer_InitTimer(POPPER_TIMER, BACKWARD_TIME);
		}
		break;

	case RackBackward:
		if (CurrentEvent.EventType == ES_TIMEOUT && CurrentEvent.EventParam == POPPER_TIMER) {
			CurrentState = Idle;
			SetPinionDuty(0, 0);
			SetPopperMode(POPPER_OFF);
		}
		break;
	}                                   // end switch on Current State
	return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/

// PWM - PE4; Direction - PE3
void SetPinionDuty(uint8_t duty, uint8_t direction) {
	if (duty == 0) {
		// Set the Duty cycle on A to 0% by programming the compare value
		// to 0. However, since the CmpADn action (set to one) wins, we also
		// need to disable the output
		HWREG( PWM0_BASE + PWM_O_1_CMPB) = 0;
		HWREG( PWM0_BASE + PWM_O_ENABLE) &= ~PWM_ENABLE_PWM3EN;
	} else if (duty > 0 && duty < 100) {
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM3EN;
		HWREG( PWM0_BASE + PWM_O_1_CMPB) = ((((PWM_TICKS_PER_S / POPPER_FREQUENCY) - 1) >> 1) * duty) / 100;
	} else if (duty == 100) {
		// Set the Duty cycle on A to 100% by programming the compare value
		// to the load value. Since the CmpADn action (set to one) wins, we get 100%
		HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM3EN;
		HWREG( PWM0_BASE + PWM_O_1_CMPB) = HWREG( PWM0_BASE + PWM_O_1_LOAD);
	}
	if (direction == FORWARD) {
		HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT2HI;
		printf("Forward\r\n");
	}
	if (direction == BACKWARD) {
		HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT2HI;
		printf("Backward\r\n");
	}
}
// PWM - PE5
void SetPopperMode(uint8_t mode) {
	if (mode == POPPER_ON) {
		HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) |= BIT3HI;
	} else if(mode == POPPER_OFF) {
		HWREG(GPIO_PORTD_BASE + (GPIO_O_DATA + ALL_BITS)) &= ~BIT3HI;
	}
}

void InitPopperPWM(void) {
	// start by enabling the clock to the PWM Module (PWM0)
	HWREG(SYSCTL_RCGCPWM) |= SYSCTL_RCGCPWM_R0;

	// enable the clock to Port B
	HWREG(SYSCTL_RCGCGPIO) |= (SYSCTL_RCGCGPIO_R1 | SYSCTL_RCGCGPIO_R3);

	// Select the PWM clock as System Clock/32
	HWREG(SYSCTL_RCC) = (HWREG(SYSCTL_RCC) & ~SYSCTL_RCC_PWMDIV_M) |
	                    (SYSCTL_RCC_USEPWMDIV | SYSCTL_RCC_PWMDIV_32);

	// make sure that the PWM module clock has gotten going
	while ((HWREG(SYSCTL_PRPWM) & SYSCTL_PRPWM_R0) != SYSCTL_PRPWM_R0)
		;

	// disable the PWM while initializing
	HWREG( PWM0_BASE + PWM_O_1_CTL ) = 0;


	// program generator B to go to 0 at rising comare B, 1 on falling compare B
	HWREG( PWM0_BASE + PWM_O_1_GENB) =
	    (PWM_1_GENB_ACTCMPBU_ZERO | PWM_1_GENB_ACTCMPBD_ONE );

	// Set the PWM period. Since we are counting both up & down, we initialize
	// the load register to 1/2 the desired total period
	HWREG( PWM0_BASE + PWM_O_1_LOAD) = ((PWM_TICKS_PER_S / POPPER_FREQUENCY) - 1) >> 1;

	// Set the initial Duty cycle on B to 0
	HWREG( PWM0_BASE + PWM_O_1_CMPB) = 0;

	// set changes to the PWM output Enables to be locally syncronized to a
	// zero count
	HWREG(PWM0_BASE + PWM_O_ENUPD) =  (HWREG(PWM0_BASE + PWM_O_ENUPD) & ~PWM_ENUPD_ENUPD3_M) | PWM_ENUPD_ENUPD3_LSYNC;

	// enable the PWM outputs
	HWREG( PWM0_BASE + PWM_O_ENABLE) |= PWM_ENABLE_PWM3EN;

	// now configure the Port B pins to be PWM outputs
	// start by selecting the alternate function for PB5
	HWREG(GPIO_PORTB_BASE + GPIO_O_AFSEL) |= BIT5HI;

	// now choose to map PWM to those pins, this is a mux value of 4 that we
	// want to use for specifying the function on bits 5
	HWREG(GPIO_PORTB_BASE + GPIO_O_PCTL) =
	    (HWREG(GPIO_PORTB_BASE + GPIO_O_PCTL) & 0xff0fffff)
	    + (4 << (5 * BITS_PER_NIBBLE));

	// Enable pin 5 on Port B for digital I/O
	HWREG(GPIO_PORTB_BASE + GPIO_O_DEN) |= BIT5HI;
	
	// Enable pins 2 & 3 on Port D for digital I/O
	HWREG(GPIO_PORTD_BASE + GPIO_O_DEN) |= (BIT2HI | BIT3HI);
	

	// make pins 5 on Port B into outputs
	HWREG(GPIO_PORTB_BASE + GPIO_O_DIR) |= BIT5HI;
	
	// Make pins 2 & 3 on Port D into outputs
	HWREG(GPIO_PORTD_BASE + GPIO_O_DIR) |= (BIT2HI | BIT3HI);

	// set the up/down count mode 00000000and enable the PWM generator
	HWREG(PWM0_BASE + PWM_O_1_CTL) |= (PWM_1_CTL_MODE | PWM_1_CTL_ENABLE);
	
	/*// start by enabling the clock to the PWM Module (PWM0) (Options are PWM0 & PWM1)
	HWREG(SYSCTL_RCGCPWM) |= SYSCTL_RCGCPWM_R0;

	// enable the clock to Port E
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R4;

	// Select the PWM clock as System Clock/32
	HWREG(SYSCTL_RCC) = (HWREG(SYSCTL_RCC) & ~SYSCTL_RCC_PWMDIV_M) |
	                    (SYSCTL_RCC_USEPWMDIV | SYSCTL_RCC_PWMDIV_32);

	// make sure that the PWM module clock has gotten going
	while ((HWREG(SYSCTL_PRPWM) & SYSCTL_PRPWM_R0) != SYSCTL_PRPWM_R0)
		;

	// disable the PWM while initializing (Generator 2 on PWM0 for channels 5 & 6)
	HWREG( PWM0_BASE + PWM_O_2_CTL ) = 0;

	// program generator A to go to 0 at rising comare A, 1 on falling compare A
	HWREG( PWM0_BASE + PWM_O_2_GENA) =
	    (PWM_2_GENA_ACTCMPAU_ZERO | PWM_2_GENA_ACTCMPAD_ONE );

	// program generator B to go to 0 at rising comare B, 1 on falling compare B
	HWREG( PWM0_BASE + PWM_O_2_GENB) =
	    (PWM_2_GENB_ACTCMPBU_ZERO | PWM_2_GENB_ACTCMPBD_ONE );


	// Set the PWM period. Since we are counting both up & down, we initialize
	// the load register to 1/2 the desired total period
	HWREG( PWM0_BASE + PWM_O_2_LOAD) = ((PWM_TICKS_PER_S / POPPER_FREQUENCY) - 1) >> 1;

	// Set the initial Duty cycle on A to 0
	HWREG( PWM0_BASE + PWM_O_2_CMPA) = 0;

	// Set the initial Duty cycle on B to 0
	HWREG( PWM0_BASE + PWM_O_2_CMPB) = 0;

	// set changes to the PWM output Enables to be locally syncronized to a
	// zero count
	HWREG(PWM0_BASE + PWM_O_ENUPD) =  (HWREG(PWM0_BASE + PWM_O_ENUPD) &
	                                   ~(PWM_ENUPD_ENUPD2_M | PWM_ENUPD_ENUPD3_M)) |
	                                  (PWM_ENUPD_ENUPD2_LSYNC | PWM_ENUPD_ENUPD3_LSYNC);

	// enable the PWM outputs
	HWREG( PWM0_BASE + PWM_O_ENABLE) |= (PWM_ENABLE_PWM4EN & PWM_ENABLE_PWM5EN);

	// now configure the Port B pins to be PWM outputs
	// start by selecting the alternate function for PB5
	HWREG(GPIO_PORTE_BASE + GPIO_O_AFSEL) |= (BIT5HI | BIT4HI);

	// now choose to map PWM to those pins, this is a mux value of 4 that we
	// want to use for specifying the function on bits 5
	HWREG(GPIO_PORTE_BASE + GPIO_O_PCTL) =
	    (HWREG(GPIO_PORTE_BASE + GPIO_O_PCTL) & 0xff00ffff)
	    + (4 << (5 * BITS_PER_NIBBLE)) + (4 << (4 * BITS_PER_NIBBLE));;

	// Enable pins 3, 4, & 5 on Port B for digital I/O
	HWREG(GPIO_PORTE_BASE + GPIO_O_DEN) |= (BIT5HI | BIT4HI | BIT3HI);

	// make pins 3, 4, & 5 on Port B into outputs
	HWREG(GPIO_PORTE_BASE + GPIO_O_DIR) |= (BIT5HI | BIT4HI | BIT3HI);

	// set the up/down count mode and enable the PWM generator
	HWREG(PWM0_BASE + PWM_O_2_CTL) |= (PWM_2_CTL_MODE | PWM_2_CTL_ENABLE);*/
	
	
}
