/****************************************************************************
 Module
   RX_SM.c

 Revision
   1.0.1

 Description
   Module for handling reception of UART communication from the XBee on the TIVA

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 05/07/15 21:09 cpb      ported over from MPLAB PIC project
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "RX_SM.h"
#include "Tiva_XBee_Comm.h"

#include <stdint.h>

#include "inc/hw_memmap.h"
#include "inc/hw_timer.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "inc/hw_uart.h"
#include "bitdefs.h"

/*----------------------------- Module Defines ----------------------------*/

#define UART_RX_DEBUG_FLAG  1
#define UART_RX_DEBUG if(UART_RX_DEBUG_FLAG) printf

// Maximum length of data in a packet
#define DATA_MAX_LENGTH 15

// time in ms the SM can stay in a state without receiving data before terminating the receive process
#define RX_ERROR_TIMEOUT 100

#define API_ID_INDEX 0

// Transmit Status indexes
#define FRAME_ID_INDEX 			1
#define TRANS_STATUS_INDEX 	2

// Packet Received indexes
#define SOURCE_ADDR_MSB_INDEX 	1
#define SOURCE_ADDR_LSB_INDEX 	2
#define SIGNAL_STR_INDEX				3
#define OPTIONS_INDEX						4


#define TRANSMIT_STATUS 0x89
#define SUCCESS 0

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

void PrintReceivedData(void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static RXState_t CurrentState;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;

// Number of data bytes in the message
static uint8_t MessageLength;

// Number of data bytes received
static uint8_t NumBytesReceived;

// Array in which data bytes are stored
static uint8_t DataReceivedArray[DATA_MAX_LENGTH];

// Running count of the value of all data bytes; when added to
static uint8_t CheckSum;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitRX_SM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitRX_SM ( uint8_t Priority )
{
	MyPriority = Priority;

	CurrentState = WaitingFor7E;

	// Enable RX interrupts

	return true;
}

/****************************************************************************
 Function
     PostRX_SM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostRX_SM( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunRX_SM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunRX_SM( ES_Event ThisEvent )
{
	ES_Event ReturnEvent;
	ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

	switch (CurrentState) {
	case WaitingFor7E:
		if (ThisEvent.EventType == RX_EVENT) {
			uint8_t ReceivedByte = (uint8_t)ThisEvent.EventParam;

			// if the 'start message' byte was received (0x7E)
			if (ReceivedByte == 0x7E) {
				CurrentState = WaitingForMSB;

				// init the safety timer
				ES_Timer_InitTimer(RX_TIMER, RX_ERROR_TIMEOUT);
			}
		}
		break;

	case WaitingForMSB:
		if (ThisEvent.EventType == RX_EVENT) {
			uint8_t ReceivedByte = (uint8_t)ThisEvent.EventParam;

			// messages are short enough the MSB should always be 0x00
			if (ReceivedByte == 0x00) {
				CurrentState = WaitingForLSB;

				// init the safety timer
				ES_Timer_InitTimer(RX_TIMER, RX_ERROR_TIMEOUT);
			}
		}
		break;

	case WaitingForLSB:
		if (ThisEvent.EventType == RX_EVENT) {
			// the lsb of the packet will contain our entire message length
			MessageLength = (uint8_t)ThisEvent.EventParam;

			// reset numBytesReceived and the CheckSum before data starts coming in
			NumBytesReceived = 0;
			CheckSum = 0xFF;

			CurrentState = ReceivingData;

			// init the safety timer
			ES_Timer_InitTimer(RX_TIMER, RX_ERROR_TIMEOUT);
		}
		break;

	case ReceivingData:
		if (ThisEvent.EventType == RX_EVENT) {
			uint8_t ReceivedByte = (uint8_t)ThisEvent.EventParam;

			// put the new byte in the data array
			DataReceivedArray[NumBytesReceived++] = ReceivedByte;

			// subtract the received byte from the checksum; the final value, added to the
			// last byte in the packet should be 0xFF
			CheckSum -= ReceivedByte;

			if (NumBytesReceived == MessageLength) {
				CurrentState = WaitingForCheckSum;
			}

			// init the safety timer
			ES_Timer_InitTimer(RX_TIMER, RX_ERROR_TIMEOUT);
		}
		break;

	case WaitingForCheckSum:
		if (ThisEvent.EventType == RX_EVENT) {
			ES_Event PostEvent;
			uint8_t ReceivedByte = (uint8_t)ThisEvent.EventParam;

			// if the message checksum is equal to our computed one
			if (ReceivedByte == CheckSum) {
				CurrentState = WaitingFor7E;

				// post an event indicating a valid packet was received
				if (DataReceivedArray[API_ID_INDEX] == TRANSMIT_STATUS) {
					if (DataReceivedArray[TRANS_STATUS_INDEX] == SUCCESS) {
						PostEvent.EventType = EV_TRANSMIT_SUCCESS;
						PostTiva_XBee_Comm(PostEvent);
					} else {
						PostEvent.EventType = EV_TRANSMIT_FAILURE;
						PostTiva_XBee_Comm(PostEvent);
					}
				} else { //Receive Packet
					PostEvent.EventType = EV_PACKET_RECEIVED;
					PostTiva_XBee_Comm(PostEvent);
					PrintReceivedData();
				}
			} else {
				printf("INVALID CHECK SUM: CHECK CONNECTION\r\n");
			}
		}
		break;

	default:
		break;
	}

	// if the error timer ever fires, we've been stuck in a state for too long, and
	// we should just reset back to the initial state (WaitingFor7E)

	if (ThisEvent.EventType == ES_TIMEOUT) {
		if (ThisEvent.EventParam == RX_TIMER) {
			CurrentState = WaitingFor7E;
		}
	}                                   // end switch on Current State
	return ReturnEvent;
}

/****************************************************************************
 Function
	QueryXBeeData
 Parameters
	an integer indicating the index in the data array to be returned
 Returns
	The value in DataReceived at index
 Description
	Allows for other services/SMs to query data received by the SM
 Notes
	will be called after a 'data received' event is posted
****************************************************************************/

uint8_t *QueryXBeeData(void) {
	return DataReceivedArray;
}

/***************************************************************************
 private functions
 ***************************************************************************/

/****************************************************************************
 Function
	PrintDataArray
 Parameters
	None
 Returns
	None
 Description
	Prints the data received in a formatted manner
****************************************************************************/

void PrintReceivedData(void) {
	if (DataReceivedArray[API_ID_INDEX] == TRANSMIT_STATUS) {
		UART_RX_DEBUG("Transmit Status: ");
		UART_RX_DEBUG("[API ID: 0x%02x] ", DataReceivedArray[API_ID_INDEX]);
		UART_RX_DEBUG("[Frame ID: 0x%02x] ", DataReceivedArray[FRAME_ID_INDEX]);
		UART_RX_DEBUG("[Status: 0x%02x]\r\n", DataReceivedArray[TRANS_STATUS_INDEX]);
	} else {
		UART_RX_DEBUG("Packet Received: ");
		UART_RX_DEBUG("[API ID: 0x%02x] ", DataReceivedArray[API_ID_INDEX]);
		UART_RX_DEBUG("[Source Address: 0x%02x%02x] ", DataReceivedArray[SOURCE_ADDR_MSB_INDEX], DataReceivedArray[SOURCE_ADDR_LSB_INDEX]);
		UART_RX_DEBUG("[Signal Strength: 0x%02x] ", DataReceivedArray[SIGNAL_STR_INDEX]);
		UART_RX_DEBUG("[Options: 0x%02x] ", DataReceivedArray[OPTIONS_INDEX]);
		UART_RX_DEBUG("[Data: ");
		for (int i = 5; i < MessageLength; i++) {
			if (i == MessageLength - 1) {
				UART_RX_DEBUG("%02x]\r", DataReceivedArray[i]);
			} else {
				UART_RX_DEBUG("%02x, ", DataReceivedArray[i]);
			}
		}
	}
}
