/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "inc/hw_gpio.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_timer.h"
#include "inc/hw_nvic.h"
#include "inc/hw_pwm.h"

#include "driverlib/sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"	// Define PART_TM4C123GH6PM in project
#include "driverlib/pwm.h"
#include "inc/hw_ssi.h"
#include "bitdefs.h"
#include "stdio.h"

#include "ES_Configure.h"
#include "ES_Framework.h"
#include "SPI.h"

/*----------------------------- Module Defines ----------------------------*/
//drive motor defines
#define PWM_TICKS_PER_S 40000000/32
#define ALL_BITS (0xff<<2)

// SPI Pin defines
#define SPI_CLK BIT2HI
#define SPI_FSS BIT3HI
#define SPI_RX BIT4HI
#define SPI_TX BIT5HI

#define CPDVSR 		4
#define SCR 			9


/*---------------------------- Module Variables ---------------------------*/
static int8_t ZOmega = 0;
static uint8_t GyroFlag = 0;

/*---------------------------- Module Functions ---------------------------*/

/****************************************************************************
 Function
    SPIInterruptResponse

 Parameters
   None

 Returns
   None

 Notes
	 Interrupt response routine for updating the rotational velocity provided by the Bosch BNO055
	 sensor. The Arduino Uno (master) sends updates every 50 ms and the Tiva (slave) receives
	 them. To post a service from the response routine, a flag is raised that is checked in 
	 EventCheckers, and an event is posted to IO_Service from there.

 Author
		 B. Allen Freund, 05/15/15, 15:15
****************************************************************************/
void SPIInterruptResponse(void) {
	ZOmega = HWREG(SSI0_BASE + SSI_O_DR);
	GyroFlag = 1;
}

uint8_t QueryGyroFlag(void) {
	return GyroFlag;
}

void ClearGyroFlag(void) {
	GyroFlag = 0;
}

int8_t QueryGyro(void) {
	return ZOmega;
}

void InitSPI(void) {
	// Enable the clock to the GPIO port A
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R0;

	// Enable the clock to SSI module
	HWREG(SYSCTL_RCGCSSI) |= SYSCTL_RCGCSSI_R0;

	// Wait for the GPIO port to be ready
	while ((HWREG(SYSCTL_PRGPIO) & SYSCTL_PRGPIO_R0) != SYSCTL_PRGPIO_R0);

	// Program the GPIO to use the alternate functions on the SSI pins
	HWREG(GPIO_PORTA_BASE + GPIO_O_AFSEL) |= (SPI_CLK | SPI_FSS | SPI_RX | SPI_TX);

	// Set mux position in GPIOPCTL to select the SSI use of the pins
	HWREG(GPIO_PORTA_BASE + GPIO_O_PCTL) =
	    ((HWREG(GPIO_PORTA_BASE + GPIO_O_PCTL) & 0xff0000ff) | 0x00222200);

	//Enable open drain select on slave select line
	//HWREG(GPIO_PORTA_BASE+GPIO_O_ODR) |= SPI_FSS;

	// Program the port lines for digital I/O
	HWREG(GPIO_PORTA_BASE + GPIO_O_DEN) |= (SPI_CLK | SPI_FSS | SPI_RX | SPI_TX);

	// Program the required data directions on the port lines
	HWREG(GPIO_PORTA_BASE + GPIO_O_DIR) &= ~(SPI_RX | SPI_FSS | SPI_CLK);

	// If using SPI mode 3, program the pull-up on the clock line
	//HWREG(GPIO_PORTA_BASE+GPIO_O_PUR) |= (SPI_CLK);//| SPI_RX);

	// Wait for the SSI0 to be ready
	while ((HWREG(SYSCTL_PRSSI) & SYSCTL_PRSSI_R0) != SYSCTL_PRSSI_R0);

	// Make sure the the SSI is disabled before programming mode bits
	HWREG(SSI0_BASE + SSI_O_CR1) &= ~SSI_CR1_SSE;

	// Select slave mode
	HWREG(SSI0_BASE + SSI_O_CR1) |= SSI_CR1_MS;
	//HWREG(SSI0_BASE+SSI_O_CR1) |= SSI_CR1_EOT;

	// Configure the SSI clock source to the system clock
	//HWREG(SSI0_BASE+SSI_O_CC) = ((HWREG(SSI0_BASE+SSI_O_CC) & ~SSI_CC_CS_M) + SSI_CC_CS_SYSPLL);

	// Configure the clock pre-scaler
	//HWREG(SSI0_BASE+SSI_O_CPSR) &= ~SSI_CPSR_CPSDVSR_M;
	//HWREG(SSI0_BASE+SSI_O_CPSR) |= CPDVSR;

	// Configure clock rate (SCR), phase & polarity (SPH, SPO), mode (FRF), data size (DSS)
	//HWREG(SSI0_BASE+SSI_O_CR0) &= ~SSI_CR0_SCR_M;
	//HWREG(SSI0_BASE+SSI_O_CR0) |=  SCR<<8;

	//HWREG(SSI0_BASE+SSI_O_CR0) |= (SSI_CR0_SPH | SSI_CR0_SPO | SSI_CR0_FRF_MOTO | SSI_CR0_DSS_8);
	HWREG(SSI0_BASE + SSI_O_CR0) &= ~SSI_CR0_FRF_M;
	HWREG(SSI0_BASE + SSI_O_CR0) |= SSI_CR0_FRF_MOTO; // mode
	HWREG(SSI0_BASE + SSI_O_CR0) &= ~SSI_CR0_SPH; // phase
	HWREG(SSI0_BASE + SSI_O_CR0) &= ~SSI_CR0_SPO; // polarity
	HWREG(SSI0_BASE + SSI_O_CR0) = ((HWREG(SSI0_BASE + SSI_O_CR0) & ~SSI_CR0_DSS_M) + SSI_CR0_DSS_8); // DSS

	// Locally enable interrupts on RXIM
	HWREG(SSI0_BASE + SSI_O_IM) |= SSI_IM_RXIM;

	// Chart on page 104 in Tiva Datasheet
	HWREG(NVIC_EN0) |= BIT7HI;

	// Globally enable interrupts
	__enable_irq();

	// Make sure that the SSI is enabled for operation
	HWREG(SSI0_BASE + SSI_O_CR1) |= SSI_CR1_SSE;
}
