/****************************************************************************
 Module
   TX_SM.c

 Revision
   1.0.1

 Description
   This is a  file for implementing flat state machines under the
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTX_SM()
 10/23/11 18:20 jec      began conversion from SMTX_.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "RX_SM.h"
#include "TX_SM.h"

#include <stdint.h>

#include "inc/hw_memmap.h"
#include "inc/hw_timer.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "inc/hw_uart.h"

#include "bitdefs.h"

/*----------------------------- Module Defines ----------------------------*/

#define UART_TX_DEBUG_FLAG 0
#define UART_TX_DEBUG if(UART_TX_DEBUG_FLAG) printf

// Max number of bytes (all, not just data) in a packet
#define PACKET_MAX_LENGTH 20

// Start delimiter for communication with the XBee
#define START_DELIM 0x7E

// Message length will never be high enough to use the MSB
#define DEFAULT_MSB 0x00

// API Identifier for send packets
#define SEND_API_IDEN 0x01

// Communication protocol currently doesn't make use of frame ID, so always use 0x01
#define BASE_FRAME_ID 0x01

// No options enabled
#define OPTION_BYTE 0x00

#define TX_FLAG HWREG(UART1_BASE + UART_O_MIS) & UART_MIS_TXMIS

#define RX_FLAG HWREG(UART1_BASE + UART_O_MIS) & UART_MIS_RXMIS

#define START_DELIM_INDEX 	0
#define MSB_INDEX 					1
#define LSB_INDEX 					2
#define API_ID_INDEX				3
#define FRAME_ID_INDEX     	4
#define	ADDR_MSB_INDEX			5
#define ADDR_LSB_INDEX			6
#define OPTION_INDEX				7


/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

void PrintSentData(void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static TXState_t CurrentState;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;

// Length of packet to send (including all bytes)
static uint8_t PacketLength;

// Count variable that keeps track of number of bytes transmitted
static uint8_t NumBytesSent;

// Array that stores all bytes in the packet
static uint8_t DataToSendArray[PACKET_MAX_LENGTH];

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitTX_SM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitTX_SM ( uint8_t Priority )
{
	MyPriority = Priority;

	CurrentState = WaitingToSend;

	NumBytesSent = 0;

	return true;
}

/****************************************************************************
 Function
     PostTX_SM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostTX_SM( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTX_SM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunTX_SM( ES_Event ThisEvent )
{
	ES_Event ReturnEvent;
	ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

	switch (CurrentState) {
	case WaitingToSend:
		if (ThisEvent.EventType == SEND_PACKET) {
			//PrintSentData();
			CurrentState = SendingData;

			// write the first data byte to the transmit register
			HWREG(UART1_BASE + UART_O_DR) = DataToSendArray[0];

			NumBytesSent++;

			// enable the transmit interrupt mask
			HWREG(UART1_BASE + UART_O_IM) |= UART_IM_TXIM;
		}
		break;

	case SendingData:
		// Not sure if anything should be done here
		break;

	default:
		break;
	}
	return ReturnEvent;
}

/****************************************************************************
 Function
	LoadXBeeData
 Parameters
	uint8_t DataArray to be sent; NumDataBytes indicating the length of DataArray; a uint16_t DestAddr to send to
 Returns
	void
 Description
	Allows for higher level services/SMs to queue up data to be sent
 Notes
	Will be followed by a SEND_PACKET Event
****************************************************************************/
void LoadXBeeData(uint8_t DataArray[], uint8_t NumDataBytes, uint16_t DestAddr) {
	uint8_t CheckSum = 0xFF;
	PacketLength = NumDataBytes + 9;

	DataToSendArray[START_DELIM_INDEX] = START_DELIM;

	DataToSendArray[MSB_INDEX] = DEFAULT_MSB;
	DataToSendArray[LSB_INDEX] = NumDataBytes + 5;

	DataToSendArray[API_ID_INDEX] = SEND_API_IDEN;
	CheckSum -= SEND_API_IDEN;

	DataToSendArray[FRAME_ID_INDEX] = BASE_FRAME_ID;
	CheckSum -= BASE_FRAME_ID;

	DataToSendArray[ADDR_MSB_INDEX] = DestAddr >> 8;
	CheckSum -= DestAddr >> 8;

	DataToSendArray[ADDR_LSB_INDEX] = DestAddr;
	CheckSum -= DestAddr;

	DataToSendArray[OPTION_INDEX] = OPTION_BYTE;
	CheckSum -= OPTION_BYTE;

	for (int i = 0; i < NumDataBytes; i++) {
		DataToSendArray[8 + i] = DataArray[i];
		CheckSum -= DataArray[i];
	}
	DataToSendArray[NumDataBytes + 8] = CheckSum;
}

/****************************************************************************
 Function
	UARTInterruptResponse
 Parameters
	N/A
 Returns
	N/A
 Description
	transmits UART data
 Notes
****************************************************************************/

void UARTInterruptResponse(void) {

	if (TX_FLAG) {
		// There's room in the transmit register, write the new byte (clearing TXRIS and TXMIS)
		HWREG(UART1_BASE + UART_O_DR) = DataToSendArray[NumBytesSent++];

		// if we just wrote the last byte
		if (NumBytesSent == PacketLength) {
			// disable transmit interrupts
			HWREG(UART1_BASE + UART_O_IM) &= ~UART_IM_TXIM;

			PrintSentData();
			// reset the number of bytes sent after transmit completed
			NumBytesSent = 0;
			CurrentState = WaitingToSend;
		}
	}

	if (RX_FLAG) {
		// post a new event to the Receive SM with the received value from the UART register
		// as the event parameter. Reading RCREG clears RXRIS and RXMIS
		ES_Event CommEvent;
		CommEvent.EventType = RX_EVENT;
		CommEvent.EventParam = (uint8_t)HWREG(UART1_BASE + UART_O_DR);
		PostRX_SM(CommEvent);
	}
}

/***************************************************************************
 private functions
 ***************************************************************************/

/****************************************************************************
 Function
	PrintSentData
 Parameters
	None
 Returns
	None
 Description
	Prints the data sent in a formatted manner
****************************************************************************/
void PrintSentData(void) {
	UART_TX_DEBUG("Sent Data: ");
	UART_TX_DEBUG("[API ID: 0x%02x] ", DataToSendArray[API_ID_INDEX]);
	UART_TX_DEBUG("[Frame ID: 0x%02x] ", DataToSendArray[FRAME_ID_INDEX]);
	UART_TX_DEBUG("[Address: 0x%02x%02x] ", DataToSendArray[ADDR_MSB_INDEX], DataToSendArray[ADDR_LSB_INDEX]);
	UART_TX_DEBUG("[Options: 0x%02x] ", DataToSendArray[OPTION_INDEX]);

	for (int i = 8; i < PacketLength - 2; i++) {
		if (i == PacketLength - 2) {
			UART_TX_DEBUG("%02x]\r\n\n", DataToSendArray[i]);
		} else {
			UART_TX_DEBUG("%02x, ", DataToSendArray[i]);
		}
	}
}
