/****************************************************************************
 Module: Tiva_XBee_Comm.c

 Description
    Module for handling communication between PIC and the XBee
****************************************************************************/

/*----------------------------- Include Files -----------------------------*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_Timers.h"
#include "ES_Queue.h"
#include <stdio.h>
#include <stdint.h>

#include "inc/hw_memmap.h"
#include "inc/hw_timer.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "inc/hw_uart.h"
#include "inc/hw_gpio.h"
#include "bitdefs.h"

#include "driverlib/gpio.h"

#include "Tiva_XBee_Comm.h"
#include "TX_SM.h"
#include "RX_SM.h"
#include "IO_Service.h"
#include "SPI.h"
#include "PopperSM.h"

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

void EnableUART(void);
void Print8BitBinary(uint8_t NumToPrint);
uint16_t ExtractSourceAddress(uint8_t *ReceiveData);
void UpdateOutputs(uint8_t *ReceiveData);
void SendStatusPacket(void);
void SendAckPacket(void);
void Unpair(void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match that of enum in header file
static XBeeState_t CurrentState;

// everybody needs a local store for the service's priority too!
static uint8_t MyPriority;

//XBee address of paired device
static uint16_t PairedAddr = 0x0000;
const uint8_t MyAddr = 7;

static uint8_t StatusPacket[] = {0x04, 0};
static uint8_t PairPacket[] = {0x02, 0x01};

static uint16_t LastMI6Addr;

static uint16_t PairingStartTime;

static bool RecentlyMindControlled = false;

// Debug flags
#define XBEE_COMM_DEBUG_FLAG  1
#define XBEE_COMM_DEBUG if(XBEE_COMM_DEBUG_FLAG) printf

// UART defines
#define BAUD_IBRD 260
#define BAUD_FBRD 27

#define UART_TX GPIO_PIN_1
#define UART_RX GPIO_PIN_0

#define ALL_BITS (0xff<<2)

// Indexes for Received Data

// Packet indexes
#define ADDR_MSB_INDEX 		1
#define ADDR_LSB_INDEX 		2
#define RSSI_INDEX				3
#define OPTIONS_INDEX  		4

// Protocol defined indexes
#define HEADER_INDEX  		5

// Protocol data indexes
#define DEST_ADDR_INDEX		6

// Transmit control message indexes
// BE CAREFUL - these are indexes into the cmdData chunk of the packet only
// Not the frameData part of the packet
#define THRUST_INDEX   		6
#define ORIENT_INDEX   		7
#define ACTION_INDEX   		8
#define EXA_INDEX	   			9
#define EXB_INDEX	   			10
#define EXC_INDEX	   			11

// Control message bit values
#define UNPAIR_BIT				0x80
#define BRAKE_BIT					0x02
#define POP_COMMAND_BIT		0x01

// Header Values
#define CONTROL_HEADER		0x03

// Options Byte Values
#define ADDR_BROADCAST 		2


// Transmit indexes (status packet)
#define MIND_INDEX				1

// Mind Control bit value
#define MIND_CONTROLLED		1
#define FREE							0

// Control values
#define UNPAIR_VALUE 0x80

// Timer Values
#define COMM_TIMEOUT 			1000
#define BALLOON_POPPED_DELAY 10000

// Lift fan PWM constants
#define BRAKE 0
#define DRAG 1
#define DRIVE 2
#define DRAG_CUTOFF 40

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitTiva_XBee_Comm
 Parameters
     uint8_t Priority
 Returns
     boolean, False if error in initialization, True otherwise
 Description
     Saves away the priority assigned to this service and posts an ES_INT
     event to this state machine
****************************************************************************/
bool InitTiva_XBee_Comm(uint8_t Priority )
{
	CurrentState = Unpaired;

	EnableUART();
	InitSPI();
	
	// Enable the clock to port D
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R3;

	// make sure that the Port D module clock has gotten going
	while ((HWREG(SYSCTL_PRGPIO) & SYSCTL_PRGPIO_R3) != SYSCTL_PRGPIO_R3)
		;

	// Set pin 7 on Port D as digital I/O
	HWREG(GPIO_PORTD_BASE + GPIO_O_DEN) |= GPIO_PIN_7;

	// make pin 7 on Port D into an output
	HWREG(GPIO_PORTD_BASE + GPIO_O_DIR) |= GPIO_PIN_7;
	
	// Set it high to activate the XBee
	HWREG(GPIO_PORTD_BASE + GPIO_O_DATA + ALL_BITS) |= GPIO_PIN_7;

	MyPriority = Priority;
	LastMI6Addr = 0;
	return true;
}

/****************************************************************************
 Function
     PostTiva_XBee_Comm
 Parameters
     ES_Event ThisEvent , the event to post to the queue
 Returns
     boolean False if the Enqueue operation failed, True otherwise
 Description
     Posts an event to the state machine's queue
****************************************************************************/
bool PostTiva_XBee_Comm( ES_Event ThisEvent )
{
	return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTiva_XBee_Comm
 Parameters
   ES_Event : the event to process
 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise
 Description
   implements a state machine for communication between XBee and PIC
 Notes
   uses nested switch/case to implement the machine.
****************************************************************************/
ES_Event RunTiva_XBee_Comm( ES_Event CurrentEvent ) {
	ES_Event ReturnEvent;

	ReturnEvent.EventType = ES_NO_EVENT; // assume no problems will occur
	switch ( CurrentState ) {

	case Unpaired:
		if (CurrentEvent.EventType == EV_PACKET_RECEIVED) {
			uint8_t *ReceiveData = QueryXBeeData();

			if (ReceiveData[OPTIONS_INDEX] & ADDR_BROADCAST) {
				uint16_t SourceAddr = ExtractSourceAddress(ReceiveData);
				uint8_t DestAddr = ReceiveData[DEST_ADDR_INDEX];

				// if it is coming from an MI6 other than the one we were last connected to
				// and is directed to our SPECTRE 
				if(DestAddr == MyAddr) {
					if (!RecentlyMindControlled || LastMI6Addr != SourceAddr) {
						CurrentState = WaitingForAckTransmitSuccess;
						PairedAddr = SourceAddr;
						SendAckPacket();
						SetLights(FRONT, ON);
						
						// Start the comm timer - Here, it keeps us from being in the
						// "WaitingForTransmitSuccess" state indefinitely
						ES_Timer_InitTimer(COMM_TIMER, COMM_TIMEOUT);

						XBEE_COMM_DEBUG("Ack Sent\r\n");
					}
				}
			}
		} else if (CurrentEvent.EventType == ES_TIMEOUT) {
			if (CurrentEvent.EventParam == COMM_TIMER) {
				RecentlyMindControlled = false;
			}
		}
		break;

	case WaitingForAckTransmitSuccess:
		if (CurrentEvent.EventType == EV_TRANSMIT_SUCCESS) {
			XBEE_COMM_DEBUG("Transmit Success\r\n");
			
			// save its address as the last address
			LastMI6Addr = PairedAddr;
			CurrentState = Paired;
			PairingStartTime = ES_Timer_GetTime();
			
			SetLiftFan(DRIVE);
			// Restart the comm timer (Now tracking control messages)
			ES_Timer_InitTimer(COMM_TIMER, COMM_TIMEOUT);
		} else if (CurrentEvent.EventType == EV_TRANSMIT_FAILURE) {
			XBEE_COMM_DEBUG("Transmit Failure\r\n");
			// If the message didn't get through resend it
			SendAckPacket();
		} else if (CurrentEvent.EventType == ES_TIMEOUT && CurrentEvent.EventParam == COMM_TIMER) {
			// If the comm timer times out without having received a transmit success message
			// Go back to the unpaired state
			XBEE_COMM_DEBUG("Comm Timeout\r\n");
			CurrentState = Unpaired;
		}
		break;

	case Paired:
		if (CurrentEvent.EventType == EV_PACKET_RECEIVED) {
			uint8_t *ReceiveData = QueryXBeeData();

			if (ReceiveData[HEADER_INDEX] == CONTROL_HEADER) {
				uint16_t SourceAddr = ExtractSourceAddress(ReceiveData);

				// Ensure we are only listening to our paired device
				if (SourceAddr == PairedAddr) {
					XBEE_COMM_DEBUG("Control Message Received. Status Message Sent. Index: %i\r\n", (ES_Timer_GetTime() - PairingStartTime) / 200);
					
					if (ReceiveData[ACTION_INDEX] & UNPAIR_BIT) {
						Unpair();
					} else {
						// if we're braking turn the lift fan off
						UpdateOutputs(ReceiveData);
						SendStatusPacket();

						// start the comm timer
						ES_Timer_InitTimer(COMM_TIMER, COMM_TIMEOUT);
					}
				}
			}
		} else if (CurrentEvent.EventType == EV_TRANSMIT_FAILURE) {
			SendStatusPacket();
			XBEE_COMM_DEBUG("Status Message Re-Sent.\r\n");
		} else if (CurrentEvent.EventType == EV_TRANSMIT_SUCCESS) {
			XBEE_COMM_DEBUG("Status Message Successful.\r\n");
		} else if (CurrentEvent.EventType == ES_TIMEOUT) {
			if (CurrentEvent.EventParam == COMM_TIMER) {
				XBEE_COMM_DEBUG("Lost Connection: %i\r\n", (ES_Timer_GetTime() - PairingStartTime) / 200);
				Unpair();
			}
		} else if (CurrentEvent.EventType == EV_BALLOONPOPPED) {
				XBEE_COMM_DEBUG("Balloon Popped!\r\n");
				RecentlyMindControlled = true;
				ES_Timer_InitTimer(COMM_TIMER, BALLOON_POPPED_DELAY);
				SendStatusPacket();
				Unpair();
		}
		break;
	}
	return ReturnEvent;
}

XBeeState_t QueryXBeeState(void) {
	return CurrentState;
}

uint16_t ExtractSourceAddress(uint8_t *ReceiveData) {
	uint16_t SourceAddr = ((uint16_t)ReceiveData[ADDR_MSB_INDEX]) << 8;
	SourceAddr |= ReceiveData[ADDR_LSB_INDEX];
	return SourceAddr;
}

void UpdateOutputs(uint8_t *ReceiveData) {
	// if we're braking turn the lift fan off
	if (ReceiveData[ACTION_INDEX] & BRAKE_BIT) {
		SetLiftFan(BRAKE);
		SetLights(BRAKE, ON);
	} else {
		int Thrust = (int8_t)ReceiveData[THRUST_INDEX];
		UpdatePIDData(Thrust, ReceiveData[ORIENT_INDEX]);
		SetLights(BRAKE, OFF);
		SetLiftFan(DRIVE);
	}

	if (ReceiveData[ACTION_INDEX] & POP_COMMAND_BIT) {
		ES_Event PopCommandEvent;
		PopCommandEvent.EventType = EV_POP;
		PostPopping_SM(PopCommandEvent);
		XBEE_COMM_DEBUG("Pop Command received");
	}
}

void Unpair(void) {
	CurrentState = Unpaired;
	SetLiftFan(BRAKE);
	SetLights(FRONT, OFF);
	SetLights(BRAKE, OFF);
}

/***************************************************************************
 private functions
 ***************************************************************************/

void SendStatusPacket(void) {
	// Load the mind control status into the status packet
	StatusPacket[MIND_INDEX] = QueryBalloonMonitor();

	// Send the status packet back
	LoadXBeeData(StatusPacket, 2, PairedAddr);
	ES_Event TX_Event;
	TX_Event.EventType = SEND_PACKET;
	PostTX_SM(TX_Event);
}

void SendAckPacket(void) {
	// send the pairing ack
	LoadXBeeData(PairPacket, 2, PairedAddr);
	ES_Event TX_Event;
	TX_Event.EventType = SEND_PACKET;
	PostTX_SM(TX_Event);
}

void EnableUART(void) {
	// Enable the clock to the UART1 module
	HWREG(SYSCTL_RCGCUART) |= SYSCTL_RCGCUART_R1;

	// Wait until the UART1 module is ready
	while ((HWREG(SYSCTL_PRUART) & SYSCTL_PRUART_R1) != SYSCTL_PRUART_R1)
		;

	// Enable clock to GPIO port B
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R1;

	// wait until the peripheral ready flag is set for ports B & E
	while ((HWREG(SYSCTL_PRGPIO) & SYSCTL_PRGPIO_R1) != SYSCTL_PRGPIO_R1)
		;

	// Disable the UART (UARTEN in UARTCTL)
	HWREG(UART1_BASE + UART_O_CTL) &= ~UART_CTL_UARTEN;

	// Set the baud rate (IBRD = 260, FBRD = 27)
	HWREG(UART1_BASE + UART_O_IBRD) = (HWREG(UART1_BASE + UART_O_IBRD) & ~UART_IBRD_DIVINT_M) | BAUD_IBRD;
	HWREG(UART1_BASE + UART_O_FBRD) = (HWREG(UART1_BASE + UART_O_FBRD) & ~UART_FBRD_DIVFRAC_M) | BAUD_FBRD;

	// UARTCTL set TXE, RXE
	HWREG(UART1_BASE + UART_O_CTL) |= (UART_CTL_TXE | UART_CTL_RXE);

	// UARTLCRH set 8 bit mode (clear all other bits)
	HWREG(UART1_BASE + UART_O_LCRH) = UART_LCRH_WLEN_8;

	// Re-enable the UART
	HWREG(UART1_BASE + UART_O_CTL) |= UART_CTL_UARTEN;

	// Set B0 and B1 to use alternate functions
	HWREG(GPIO_PORTB_BASE + GPIO_O_AFSEL) |= (UART_TX | UART_RX);

	// Then, map bits 0 and 1's alternate functions to U1RX and U1TX
	// 1 is the mux value to select U1RX & U1RX; U1TX must be shifted left by 4
	HWREG(GPIO_PORTB_BASE + GPIO_O_PCTL) = (HWREG(GPIO_PORTB_BASE + GPIO_O_PCTL) & 0xffffff00) + 1 + (1 << 4);

	// Set B0 and B1 as digital pins
	HWREG(GPIO_PORTB_BASE + GPIO_O_DEN) |= (UART_TX | UART_RX);

	// Set B0 as an input
	HWREG(GPIO_PORTB_BASE + GPIO_O_DIR) &= ~UART_RX;

	// Set B1 as an output
	HWREG(GPIO_PORTB_BASE + GPIO_O_DIR) |= UART_TX;

	// Enable the open drain output for the TX pin
	// HWREG(GPIO_PORTB_BASE + GPIO_O_ODR) |= UART_TX;

	// Enable the receive interrupt locally
	HWREG(UART1_BASE + UART_O_IM) |= UART_IM_RXIM;

	// Enable the interrupts in the NVIC; Interrupt #6 so bit 6 in register 0
	// see page 104 in datasheet
	HWREG(NVIC_EN0) |= BIT6HI;

	// Enable interrupts globally
	__enable_irq();
}

void Print8BitBinary(uint8_t NumToPrint) {
	for (int i = 0; i < 8; i++) {
		if (NumToPrint & 1) {
			XBEE_COMM_DEBUG("1");
		} else {
			XBEE_COMM_DEBUG("0");
		}
		NumToPrint >>= 1;
	}
	XBEE_COMM_DEBUG("\r\n");
}
